package Client;

import java.io.*;
import java.net.*;
import java.rmi.*;
import Shared.*;
import ImageTools.KodakProducer;

/**<dl><dt>
 * Module ID: ClientCOMM.java                                               <dt>
 * Date     : 11 Avril 1997                                                 <dt>
 *                                                                          <dt>
 * Revision :                                                               <dt>
 *                                                                          <dt>
 *   001      15 Avril 1997 - Olivier Langlois                              <dt>
 *   Le tampon de is a ete augmente de facon a pouvoir contenir 2 images de <dt>
 *   64 x 64.
 * </dl>
 * @author    Olivier Langlois
 * @version   1.001
 *
 * @see ImageCmdConst
 */
public class ClientCOMM implements ImageCmdConst
{
  private Socket              ClientSocket;
  private BufferedInputStream is;
  private OutputStream        os;
  private String              ServerURL;

  /**
   * Initialise une connection avec un Serveur d'images
   *
   * @param ServeurAddress  Addresse du serveur
   * @param port            port du serveur
   */
  public ClientCOMM(InetAddress ServerAddress, int port) throws IOException
  {
    ClientSocket = new Socket( ServerAddress, port );
    ClientSocket.setTcpNoDelay(true);
    is = new BufferedInputStream(ClientSocket.getInputStream(), 8192);
    os = ClientSocket.getOutputStream();
    ServerURL              = ServerAddress.getHostName();
  }

  /**
   * Initialise une connection avec un Serveur d'images
   *
   * @param ServeurURL      URL du serveur
   * @param port            port du serveur
   */
  public ClientCOMM(String ServerURL, int port) throws IOException
  {
    ClientSocket = new Socket( ServerURL, port );
    ClientSocket.setTcpNoDelay(true);
    is = new BufferedInputStream(ClientSocket.getInputStream(), 8192);
    os = ClientSocket.getOutputStream();
    this.ServerURL = ServerURL;
  }

/******************************************************************************
 *
 *             Demande au serveur la liste des images disponibles.
 *
 * @return     String[] ImageListe
 *
 ****************************************************************************/
  public String[] getList() throws IOException
  {
    /** Envoie la commande GETLIST */
    os.write( GETLIST.getBytes() );
    System.out.println( "GETLIST envoye" );

    /** Recoit le nombre d'entrees dans la liste */
    int i = Integer.parseInt( f_readASCII() );
    String[] ImageListe = new String[i];

    /** Recoit les items de la liste. */
    for( int j = 0; j < i; j++ )
      ImageListe[j] = f_readASCII();

    /** Retourne la liste */
    return ImageListe;
  }

/******************************************************************************
 *
 *             Demande au serveur d'envoyer une image.
 *
 * Note      : ImageName doit etre une des chaines de caracteres retournees
 *             par la methode getList().
 *
 * @see        ClientCOMM#getList
 *
 * @param      ImageName (String) Nom de l'image demandee.
 *
 * @return     kodakProducer.
 *
 * @exception  java.rmi.NotBoundException
 *
 ****************************************************************************/
  public KodakProducer getImage( String ImageName ) throws IOException, NotBoundException
  {
    /** Envoie la commande GETIMAGE */
    os.write( GETIMAGE.getBytes() );

    /**
     * Pour permettre au serveur de se preparer a envoyer l'image,
     * Retrouve la reference de l'objet de type RemoteImageHeader
     * portant le meme nom que l'image par le biais du registre RMI.
     */
    RemoteImageHeader Header = (RemoteImageHeader)Naming.lookup( "//" +
                                ServerURL + "/" + ImageName );

    /** Requete l'image ImageName au serveur */
    os.write( ImageName.getBytes() );
    os.write( "\n".getBytes() );

    /** Retourne un KodakProducer */
    return new KodakProducer(Header, is);
  }

/******************************************************************************
 *
 *             Termine la communication avec le serveur.
 *
 * Parametres:
 *     Aucun.
 *
 * Valeur de retour: Aucune.
 *
 ****************************************************************************/
  public void termineCOMM()
  {
    /** Avertit le serveur que la connection sera brisee par le client */
    try
    {
      os.write( BYE.getBytes() );
    }
    catch( IOException ioe )
    {
      System.out.println( ioe.getMessage() );
    }
    /** Ferme le socket */
    try
    {
      ClientSocket.close();
    }
    catch( IOException ioe )
    {
      System.out.println( ioe.getMessage() );
    }
  }

/******************************************************************************
 *
 *             Lire la reponse du serveur.
 *
 * Parametres:
 *     Aucun.
 *
 * Valeur de retour: String.
 *
 ****************************************************************************/
  private String f_readASCII() throws IOException
  {
    char c;
    StringBuffer tampon = new StringBuffer();
    while( (c = (char) is.read()) != '\n' ) tampon.append(c);
    return tampon.toString();
  }
}
