/*
 * <applet code="ImageClient" width=384 height=356>
 * </applet>
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.applet.*;
import java.util.*;
import java.io.IOException;
import java.rmi.RemoteException;
import Client.*;
import ImageTools.*;

/**<dl><dt>
 * Module ID: ImageClient.java                 <dt>
 * Note     : Ce module necessite le JDK 1.1.1 <dt>
 * Date     : 18 Avril 1997
 * </dl>
 * @author    Olivier Langlois
 * @version   1.000
 */
public class ImageClient extends Applet implements ItemListener, Observer
{
  static final int portNumber = 10012;      //numero du port

  private ImageCanvas MapCanvas;
  private ImageCanvas PhotoCanvas;
  private ImagesChoix choix;
  private Checkbox    SobelChoix;

  // Les filtres
  private ScaleFilter scale;
  private SobelFilter sobel;

  private ImageProducer producteur;
  private KodakProducer kodakP;
  private String        currentIm;
  private boolean       PhotoFlag;

  private ClientCOMM  COMMChannel;

  public void init()
  {
    try
    {
      // Tente de rejoindre le serveur
      COMMChannel = new ClientCOMM( getCodeBase().getHost(), portNumber );

      // prepare la fenetre
      f_prepareFenetre();

      // Initialise les filtres
      scale = new ScaleFilter(2);
      sobel = new SobelFilter();

      currentIm = "";
    }
    catch( Exception _e )
    {
        System.out.println( "init error: " + _e + _e.getMessage() );
    }
  }

  public void destroy()
  {
    COMMChannel.termineCOMM();
    super.destroy();
  }

/******************************************************************************
 *
 * Nom       : f_prepareFenetre
 *
 * Utilite   : Prepare la fenetre pour l'affichage.
 *
 * Parametres:
 *     Aucun.
 *
 * Valeur de retour: Aucune.
 *
 ****************************************************************************/
  private void f_prepareFenetre() throws IOException
  {
    setLayout(null);
    int width   = Integer.parseInt(getParameter("width"));
    int height  = Integer.parseInt(getParameter("height"));

    // Initialise les elements graphiques
    MapCanvas   = new ImageCanvas(this);
    PhotoCanvas = new ImageCanvas(this);
    choix       = new ImagesChoix(COMMChannel);
    SobelChoix  = new Checkbox("Filtre de Sobel");

    // Installe les listeners
    choix.addItemListener(this);
    SobelChoix.addItemListener(this);
    MapCanvas.addMouseListener( MapCanvas );
    MapCanvas.setEnabled(false);

    // Ajoute les elements graphiques a l'interieur du applet panel.
    add(MapCanvas);
    add(PhotoCanvas);
    add(choix);
    add(SobelChoix);

    // Ajuste les dimensions
    MapCanvas.setBounds  (   0, 0, 256, 256 );
    PhotoCanvas.setBounds( 256, 0, 128, 128 );
    choix.setBounds      (   0, height - 100, width / 2, 100 );
    SobelChoix.setBounds ( width / 2, height - 100, width / 2, 100 );
  }

  public void update( Observable producteur, Object arg )
  {
    f_activate(true);
  }

  public void itemStateChanged( ItemEvent ie )
  {
    if( ie.getItemSelectable() == SobelChoix )
    {
      if( ie.getStateChange() == ItemEvent.SELECTED )
        sobel.setStateFilter(true);
      else
        sobel.setStateFilter(false);
      if( PhotoFlag )
      {
        f_activate(false);
        f_redessine(PhotoCanvas);
      }
    }
    else if( ie.getItemSelectable() == choix )
    {
      if( !ie.getItem().equals(currentIm) )
      {
        currentIm  = (String)ie.getItem();
        showStatus( "Transfert " + currentIm + "..." );
        try
        {
          // Desactive l'interface
          f_activate(false);

          // Transfer image du serveur
          producteur = kodakP = COMMChannel.getImage(currentIm);
          showStatus( "Transfert termine" );

          // Demande au producteur de nous avertit lorsqu'il termine une production.
          kodakP.addObserver(this);

          // Efface le contenue des canvas
          MapCanvas.getGraphics().clearRect( 0, 0, 256, 256 );
          PhotoCanvas.getGraphics().clearRect( 0, 0, 128, 128 );

          f_redessine(MapCanvas);

          // Prepare la source d'image pour les manipulations
          producteur = new FilteredImageSource(
                         new FilteredImageSource( kodakP, sobel ),
                       scale );
          PhotoFlag = false;
        }
        catch( Exception e )
        {
          f_activate(true);
          showStatus( "Transfert echoue : " + e.getMessage() );
        }
      }
    }
  }

/******************************************************************************
 *
 *             Dessine la matrice #matNum dans le canvas PhotoCanvas.
 *
 * @param      matNum  (int) # de matrice.
 *
 ****************************************************************************/
  public void drawMatrix( int matNum )
  {
    try
    {
      kodakP.setMatrix( matNum );
      f_activate(false);
      f_redessine( PhotoCanvas );
      PhotoFlag = true;
    }
    catch( KodakException ke )
    {
      System.out.println( "Erreur drawMatrix: " + ke + ke.getMessage() );
    }
  }

  private void f_redessine( ImageCanvas target )
  {
    // Prepare l'image
    Image im = createImage(producteur);
    target.setImage( im );
    prepareImage( im, target );
  }

  private void f_activate( boolean state )
  {
    // Active/Desactive les commandes
    choix.setEnabled(state);
    SobelChoix.setEnabled(state);
    MapCanvas.setEnabled(state);
    if( state )
      setCursor( Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR) );
    else
      setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
  }
}

