package ImageTools;

import java.awt.image.*;

/**<dl><dt>
 * Module ID: ScaleFilter.java                                                       <dt>
 * Utilite  : Filtre d'agrandissement                                                <dt>
 * Date     : 16 Avril 1997                                                          <dt>
 *                                                                                   <dt>
 * Note     : Le fonctionnement de cette classe suppose que la source porvient d'un  <dt>
 *            KodakProducer. Si ce n'est pas le cas, il se peut que le filtre        <dt>
 *            ne fonctionne pas comme desire.                                        <dt>
 *            Ce filtre ne supporte que la version int[] de setPixels().             <dt>
 *            Ne fonctionne que pour scaleFactor = 2 presentement.
 * </dl>
 * @author    Olivier Langlois
 * @version   1.000
 */
public class ScaleFilter extends ImageFilter
{
  protected int        scaleFactor;
  protected int        imgxsize, imgysize;
  protected int        imgpixels[][];
  protected ColorModel cm;
  protected int        lastSettingMax, SettingMax;

  // Constructeurs
  /**
   * Initialise un ScaleFilter
   * @param scaleFactor Facteur d'agrandissement.
   */
  public ScaleFilter( int scaleFactor )
  {
    lastSettingMax = SettingMax = 0;
    this.scaleFactor = scaleFactor;
  }

  public void setDimensions( int w, int h )
  {
    imgxsize  = w * scaleFactor;
    imgysize  = h * scaleFactor;
    imgpixels = new int[imgysize][imgxsize];
    consumer.setDimensions( imgxsize, imgysize );
  }

  public void setColorModel( ColorModel cm )
  {
    this.cm = cm;
    super.setColorModel( cm );
  }

  public void setPixels( int x1, int y1, int w, int h, ColorModel cm,
                         int pixels[], int off, int scansize )
  {
    int x, y, x2, y2, sx, sy;

    x2 = x1+w;
    y2 = y1+h;
    SettingMax = (y2 - 1)*scaleFactor;
    sy = off;

    if( scansize > w )
    {
      for( y = y1; y < y2; y++ )
      {
        sx = sy;
        for( x = x1; x < x2; x++ )
        {
          imgpixels[y*scaleFactor][x*scaleFactor] = pixels[sx++];
        }
        sy += scansize;
      }
    }
    else
    {
      for( ; x1 < x2; x1+=scansize )
      {
        for( y = y1; y < y2; y++ )
        {
          for( x = x1; x < x1 + scansize; x++ )
          {
            imgpixels[y*scaleFactor][x*scaleFactor] = pixels[sy++];
          }
        }
      }
    }
  }

  public void imageComplete( int status )
  {
    /*
     * 3 cas possibles pour scaleFactor == 2
     *
     *  1. C I C   2. I C I   3. I I I
     *     I R I      I R I      C R C
     *     C I C      I C I      I I I
     *
     * ou C :  Pixel connu
     *    I :  Pixel inconnu
     *    R :  Pixel recherche
     *
     * Cas #1 lorsque (Yr1) % scaleFactor != 0 && (Xr1) % scaleFactor != 0
     *        alors R = (P[Yr-1][Xr-1]+P[Yr-1][Xr+1]+P[Yr+1][Xr-1]+P[Yr+1][Xr-1])/4
     *
     * Cas #2 lorsque (Yr1) % scaleFactor == 0 && (Xr1) % scaleFactor != 0
     *        alors R = (P[Yr-1][Xr]+P[Yr+1][Xr])/2
     *
     * Cas #3 lorsque (Yr1) % scaleFactor != 0 && (Xr1) % scaleFactor == 0
     *        alors R = (P[Yr][Xr-1]+P[Yr][Xr+1])/2
     */

    if( status == ImageConsumer.SINGLEFRAMEDONE )
    {
      if( scaleFactor == 2 )
      {
        int lines;
        if( (lines = SettingMax - lastSettingMax) > 0 )
        {
          int s = 0;
          int[] PixelBlock;
          if( SettingMax == (imgysize - 2) )
            PixelBlock = new int[(lines+2)*imgxsize];
          else
            PixelBlock = new int[(lines+1)*imgxsize];
          for( int y = lastSettingMax; y <= SettingMax; y++ )
          {
            if( (y % 2) == 0 )
            {
              // Cas #3
              for( int x = 0; x < imgxsize-1; x++ )
              {
                if( (x % 2) == 0 )
                  PixelBlock[s++] = imgpixels[y][x];
                else
                  PixelBlock[s++] = (imgpixels[y][x-1]+imgpixels[y][x+1])/2; 
              }
              // Derniere colonne
              PixelBlock[s++] = imgpixels[y][imgxsize-2];
            }
            else
            {
              for( int x = 0; x < imgxsize; x++ )
              {
                // Derniere colonne
                if( x == (imgxsize-1) )
                  PixelBlock[s++] = (imgpixels[y-1][x-1]+imgpixels[y+1][x-1])/2;
                // Cas #2
                else if( (x % 2) == 0 )
                {
                  PixelBlock[s++] = (imgpixels[y-1][x]+imgpixels[y+1][x])/2;
                }
                // Cas #1
                else
                  PixelBlock[s++] = (imgpixels[y-1][x-1]+imgpixels[y-1][x+1]+
                                     imgpixels[y+1][x-1]+imgpixels[y+1][x+1])/4;
              }
            }
          }      // Fin de FOR each Y

          if( SettingMax == (imgysize - 2) )
          {
            // Derniere ligne
            while(s < PixelBlock.length )
            {
              PixelBlock[s++] = PixelBlock[s-(imgxsize+1)];
            }
            consumer.setPixels(0, lastSettingMax, imgxsize, lines+2, cm,
                               PixelBlock, 0, imgxsize);
            lastSettingMax = 0;
          }
          else
          {
            consumer.setPixels(0, lastSettingMax, imgxsize, lines, cm,
                               PixelBlock,0, imgxsize);
            lastSettingMax = SettingMax;
          }
          super.imageComplete( status );
        }        // Fin de IF SettingMax - lastSettingMax > 0
      }          // Fin de IF scaleFactor == 2
    }            // Fin de IF SINGLEFRAMEDONE
    else super.imageComplete( status );
  }
}
