package ImageTools;

import java.awt.image.*;
import java.util.Hashtable;

/**<dl><dt>
 * Module ID: SeuilFilter.java                                                      <dt>
 * Utilite  : Filtre de Seuillage                                                   <dt>
 * Date     : 18 Avril 1997                                                         <dt>
 *                                                                                  <dt>
 * Note     : Le fonctionnement de cette classe suppose que la source porvient d'un <dt>
 *            KodakProducer. Si ce n'est pas le cas, il se peut que le filtre       <dt>
 *            ne fonctionne pas comme desire.                                       <dt>
 *            Par exemple, ce filtre a besoin de la propriete PIX_MAX_VAL definit   <dt>
 *            l'interface KodakFltConst.                                            <dt>
 * </dl>
 * @author    Olivier Langlois
 * @version   1.000
 */
public class SeuilFilter extends ImageFilter implements KodakFltConst
{
  protected int        maxVal;
  protected int        seuilVal;

  private   boolean    actif;

  // Constructeurs
  SeuilFilter()
  {
    maxVal = 0;
    actif  = true;
  }

/******************************************************************************
 *
 *             Active/desactive le filtre.
 *
 * @param      seuilVal (float) Nouvelle valeur de seuillage de (0 a 1).
 *
 ****************************************************************************/
  public void setStateFilter( float seuilVal )
  {
    /**
     * L'intervalle valide de seuilVal est de 0  1 et reprsente le pourcentage de     
     * maxVal.
     * En ajustant la valeur de seuillage de cette facon, on libre l'utilisateur de ce
     * filtre de la responsabilite de verifier la valeur maximum valide de l'index
     * de couleur.
     */
    this.seuilVal = (int)(maxVal*(seuilVal > 1 ? 1 : (seuilVal < 0 ? 0 : seuilVal)));
  }

  public void setProperties( Hashtable props )
  {
    if( props.containsKey(PIX_MAX_VAL) )
    {
      maxVal = ((Integer)props.get(PIX_MAX_VAL)).intValue();
      actif  = true;
    }
    else actif = false;
    super.setProperties( props );
  }

  public void setPixels( int x1, int y1, int w, int h, ColorModel cm,
                         int pixels[], int off, int scansize )
  {
    if( actif )
    {
      int x, y, x2, y2, sx, sy, sz;
      int[] sentpixels = new int[w*h];

      x2 = x1+w;
      y2 = y1+h;
      sy = off;
      sz = 0;

      if( scansize > w )
      {
        for( y = y1; y < y2; y++ )
        {
          sx = sy;
          for( x = x1; x < x2; x++ )
          {
            if( pixels[sx] > seuilVal )
              sentpixels[sz++] = pixels[sx++];
            else
            {
              sentpixels[sz++] = seuilVal;
              sx++;
            }
          }
          sy += scansize;
        }
      }
      else
      {
        for( ; x1 < x2; x1+=scansize )
        {
          for( y = y1; y < y2; y++ )
          {
            for( x = x1; x < x1 + scansize; x++ )
            {
              if( pixels[sy] > seuilVal )
                sentpixels[sz++] = pixels[sy++];
              else
              {
                sentpixels[sz++] = seuilVal;
                sy++;
              }
            }
          }
        }
      }
      super.setPixels(x1, y1, w, h, cm, sentpixels, 0, w);
    }    // Fin de IF actif
    else super.setPixels(x1, y1, w, h, cm, pixels, off, scansize );
  }
}
