package ImageTools;

import java.awt.image.*;

/**<dl><dt>
 * Module ID: SobelFilter.java                                                           <dt>
 * Utilite  : Filtre de Sobel                                                            <dt>
 * Date     : 17 Avril 1997                                                              <dt>
 *                                                                                       <dt>
 * Note     : Le fonctionnement de cette classe suppose que la source porvient d'un      <dt>
 *            KodakProducer. Si ce n'est pas le cas, il se peut que le filtre            <dt>
 *            ne fonctionne pas comme desire.                                            <dt>
 *            Par exemple, ce filtre a besoin d'un ColorModel de type IndexColorModel.   <dt>
 *            Pour activer le filtre, il faut que le producteur appelle setColorModel(). 
 * </dl>
 * @author    Olivier Langlois
 * @version   1.000
 */
public class SobelFilter extends ImageFilter
{
  protected int        imgxsize, imgysize;
  protected int        imgpixels[][];
  protected ColorModel cm;
  protected int        maxVal;
  protected int        lastSettingMax, SettingMax;

  private   boolean    actif;

  // Constructeurs
  /**
   * Initialise un filtre de Sobel
   */
  public SobelFilter()
  {
    lastSettingMax = SettingMax = maxVal = 0;
    actif = false;
  }

/******************************************************************************
 *
 *             Active/desactive le filtre.
 *
 * @param      state (boolean) Nouvel etat du filtre.
 *
 ****************************************************************************/
  public void setStateFilter( boolean state )
  {
    actif = state;
  }

  public void setDimensions( int w, int h )
  {
    imgxsize  = w;
    imgysize  = h;
    imgpixels = new int[imgysize][imgxsize];
    consumer.setDimensions( imgxsize, imgysize );
  }

  public void setColorModel( ColorModel cm )
  {
    this.cm = cm;
    if( cm instanceof IndexColorModel )
    {
      maxVal = ((IndexColorModel)cm).getMapSize();
    }
    else actif = false;
      
    super.setColorModel( cm );
  }

  public void setPixels( int x1, int y1, int w, int h, ColorModel cm,
                         int pixels[], int off, int scansize )
  {
    if( actif )
    {
      int x, y, x2, y2, sx, sy;

      x2 = x1+w;
      y2 = y1+h;
      SettingMax = y2 - 1;
      sy = off;

      if( scansize > w )
      {
        for( y = y1; y < y2; y++ )
        {
          sx = sy;
          for( x = x1; x < x2; x++ )
          {
            imgpixels[y][x] = pixels[sx++];
          }
          sy += scansize;
        }
      }
      else
      {
        for( ; x1 < x2; x1+=scansize )
        {
          for( y = y1; y < y2; y++ )
          {
            for( x = x1; x < x1 + scansize; x++ )
            {
              imgpixels[y][x] = pixels[sy++];
            }
          }
        }
      }
    }    // Fin de IF actif
    else super.setPixels(x1, y1, w, h, cm, pixels, off, scansize );
  }

  public void imageComplete( int status )
  {
    if( actif )
    {
      if( status == ImageConsumer.SINGLEFRAMEDONE )
      {
        int lines;
        if( (lines = SettingMax - lastSettingMax) > 1 )
        {
          int e1, e2;
          int s = (lastSettingMax == 0 ? imgxsize : 0);
          int[] PixelBlock = new int[(lines+1)*imgxsize];
          for( int y = (lastSettingMax == 0 ? 1 : lastSettingMax); y < SettingMax; y++ )
          {
            s++;
            for( int x = 1; x < imgxsize-1; x++ )
            {
              e1=Math.abs((imgpixels[y+1][x-1]+2*imgpixels[y+1][x]+imgpixels[y+1][x+1]) - 
                          (imgpixels[y-1][x-1]+2*imgpixels[y-1][x]+imgpixels[y-1][x+1]));

              e2=Math.abs((imgpixels[y-1][x-1]+2*imgpixels[y][x-1]+imgpixels[y+1][x-1]) - 
                          (imgpixels[y-1][x+1]+2*imgpixels[y][x+1]+imgpixels[y+1][x+1]));

              PixelBlock[s++] = ((e1 + e2) > maxVal ? maxVal : e1 + e2);
            }
            // Derniere colonne
            PixelBlock[s++] = PixelBlock[s-1];
            // Premiere colonne
            PixelBlock[s-imgxsize] = PixelBlock[s-(imgxsize-1)];
          }      // Fin de FOR each Y

          if( lastSettingMax == 0 )
          {
            // Premiere ligne
            PixelBlock[0] = PixelBlock[imgxsize];
            for( int x = 1; x < imgxsize - 2; x++ )
            {
              PixelBlock[x] = PixelBlock[x+imgxsize];
            }
            PixelBlock[imgxsize-1] = PixelBlock[2*(imgxsize-1)];
          }

          if( SettingMax == (imgysize - 1) )
          {
            // Derniere ligne
            while(s < PixelBlock.length )
            {
              PixelBlock[s++] = PixelBlock[s-(imgxsize+1)];
            }
            consumer.setPixels(0, lastSettingMax, imgxsize, lines+1, cm,
                               PixelBlock, 0, imgxsize);
            lastSettingMax = 0;
          }
          else
          {
            consumer.setPixels(0, lastSettingMax, imgxsize, lines, cm,
                               PixelBlock,0, imgxsize);
            lastSettingMax = SettingMax;
          }
          super.imageComplete( status );
        }          // Fin de IF SettingMax - lastSettingMax > 0
      }            // Fin de IF SINGLEFRAMEDONE
      else super.imageComplete( status );
    }              // Fin de IF actif
    else super.imageComplete( status );
  }
}
