package Serveur;

import java.net.*;
import java.io.*;
import java.util.Enumeration;
import Shared.*;
import ImageServeur;

/**<dl><dt>
 * Module ID: ImageServeurThread.java <dt>
 * Date     : 27 Mars 1997
 *
 * @author    Olivier Langlois
 * @version   1.000
 */
public class ImageServeurThread extends Thread implements ImageCmdConst
{
  public static ImageServeur Serveur;

  private Socket socket = null;
  private InputStream          is;
  private BufferedOutputStream os;

  // Constructeur
  /**
   * Initialise une nouvelle thread de communication
   * @param socket socket du client
   * @param Connections ThreadGroup des threads de communications.
   */
  public ImageServeurThread(Socket socket, ThreadGroup Connections)
  {
    /*
     * Initialise le thread
     * Identificateur unique pour chaque thread soit
     * l'addresse du client (HOSTNAME + #PORT).
     */
    super(Connections, socket.getInetAddress().getHostName() + ":" + socket.getPort());
    this.socket = socket;
  }

/******************************************************************************
 *
 *             Recoit et repond aux commandes des clients.
 *
 ****************************************************************************/
  public void run()
  {
    byte[] c = new byte[CMDLEN]; // Buffer de reception
    String cmd;

    try
    {
      is = socket.getInputStream();
      os = new BufferedOutputStream(socket.getOutputStream());

      f_ecris( "Connection etablie" );

      for(;;)
      {
        // Recoit la commande
        is.read(c);
        cmd = new String(c);

        if( cmd.equals(UPDATE) )
        {
          f_ecris( "Reception de la commande : " + UPDATE );
          // Appelle la methode update du serveur
          // Retourne a l'usager la liste des images disponibles
        }
        else if( cmd.equals(GETLIST) )
        {
          f_ecris( "Reception de la commande : " + GETLIST );

          // Retourne a l'usager le nombre d'entrees dans la liste.
          byte[] data = Integer.toString(Serveur.getImageTableSize()).getBytes();
          os.write( data, 0, data.length );
          os.write("\n".getBytes());

          // Retourne a l'usager la liste des images disponibles
          for( Enumeration e = Serveur.getImageTableKeys(); e.hasMoreElements(); )
          {
            data = ((String)e.nextElement()).getBytes();
            os.write( data, 0, data.length );
            os.write("\n".getBytes());
          }
          os.flush();
        }
        else if( cmd.equals(GETIMAGE) )
        {
          f_ecris( "Reception de la commande : " + GETIMAGE );
          // Attend le nom du fichier.
          String FileName = f_readASCII();
          
          // Par la suite, envoie le fichier binaire.
          ImageHeader Header    = Serveur.getImageHeader( FileName );
          int numIm             = Header.getnumImages();
          int X                 = Header.getmatrixSize_X();
          int Y                 = Header.getmatrixSize_Y();
          int numPix            = Header.getnumBytesPerPix();
          InputStream input     = new FileInputStream(ImageHeader.DATA_DIR + FileName);
          byte imageByteArray[] = new byte[numIm * X * Y * numPix];
          input.read(imageByteArray, 0, imageByteArray.length);
          input.close();
          os.write(imageByteArray, 0, imageByteArray.length);
          os.flush();
        }
        else if ( cmd.equals(BYE) )
        {  
          f_ecris( "Termine la session" );
          break;
        }
        else
        {
          f_ecris( "Commande inconnue : " + cmd );
        }
      }
      os.close();
      is.close();
      socket.close();

      // Si ce thread est le dernier, tente de terminer le programme. 
      if( Thread.currentThread().activeCount() == 1 )
        Serveur.termineProgramme();
    }
    catch (IOException e)
    {
      f_ecris( e + e.getMessage() );
    }
  }

/******************************************************************************
 *
 * Nom       : f_ecris
 *
 * Utilite   : Envoie des messages au TextArea du serveur.
 *
 * Parametres:
 *     message (String) Message a afficher.
 *
 * Valeur de retour: Aucune.
 *
 ****************************************************************************/
  private void f_ecris( String message )
  {
    if( Serveur.isListening() )
    {
      Serveur.display.append( Thread.currentThread().getName() + "\n" +
                              message + "\n" );
    }
  }
/******************************************************************************
 *
 * Nom       : f_readASCII
 *
 * Utilite   : Lire la requete du client.
 *
 * Parametres:
 *     Aucun.
 *
 * Valeur de retour: Aucune.
 *
 ****************************************************************************/
  private String f_readASCII() throws IOException
  {
    char c;
    StringBuffer tampon = new StringBuffer();
    while( (c = (char) is.read()) != '\n' ) tampon.append(c);
    return tampon.toString();
  }
}

